@@ -1,7 +1,7 @@
 from dataclasses import dataclass
+from typing import List
 from typing import Union
 from enum import Enum
-from typing import List
 
 
 class ActivationsEnum(Enum):
@@ -196,14 +196,17 @@ class NodeType(Enum):
     Nadam = "Nadam"
     RMSprop = "RMSprop"
     SGD = "SGD"
+    OnnxErf = "OnnxErf"
     OnnxHardSigmoid = "OnnxHardSigmoid"
+    OnnxReduceMean = "OnnxReduceMean"
+    OnnxSqrt = "OnnxSqrt"
     Dataset = "Dataset"
     RepresentationBlock = "RepresentationBlock"
     GroundTruth = "GroundTruth"
     CustomLoss = "CustomLoss"
     Visualizer = "Visualizer"
-    Lambda = "Lambda"
     TFOpLambda = "TFOpLambda"
+    Lambda = "Lambda"
     SlicingOpLambda = "SlicingOpLambda"
     Repeat = "Repeat"
     Variable = "Variable"
@@ -1640,6 +1643,11 @@ class SGD:
     type: NodeType
 
 
+@dataclass
+class OnnxErf:
+    type: NodeType
+
+
 @dataclass
 class OnnxHardSigmoid:
     alpha: float
@@ -1647,6 +1655,18 @@ class OnnxHardSigmoid:
     type: NodeType
 
 
+@dataclass
+class OnnxReduceMean:
+    axes: List[int]
+    keepdims: bool
+    type: NodeType
+
+
+@dataclass
+class OnnxSqrt:
+    type: NodeType
+
+
 @dataclass
 class Dataset:
     type: NodeType
@@ -1673,12 +1693,12 @@ class Visualizer:
 
 
 @dataclass
-class Lambda:
+class TFOpLambda:
     type: NodeType
 
 
 @dataclass
-class TFOpLambda:
+class Lambda:
     type: NodeType
 
 
@@ -1727,6 +1747,6 @@ class NodeDataTypes:
                  ZeroPadding1D, ZeroPadding2D, ZeroPadding3D, BinaryCrossentropy, BinaryFocalCrossentropy, 
                  CategoricalCrossentropy, CategoricalHinge, CosineSimilarity, Hinge, Huber, KLDivergence, LogCosh, 
                  MeanAbsoluteError, MeanAbsolutePercentageError, MeanSquaredError, MeanSquaredLogarithmicError, Poisson, 
-                 SquaredHinge, Adadelta, Adagrad, Adam, Adamax, Ftrl, Nadam, RMSprop, SGD, OnnxHardSigmoid, Dataset, 
-                 RepresentationBlock, GroundTruth, CustomLoss, Visualizer, Lambda, TFOpLambda, SlicingOpLambda, Repeat, 
-                 Variable, Gather]
+                 SquaredHinge, Adadelta, Adagrad, Adam, Adamax, Ftrl, Nadam, RMSprop, SGD, OnnxErf, OnnxHardSigmoid, 
+                 OnnxReduceMean, OnnxSqrt, Dataset, RepresentationBlock, GroundTruth, CustomLoss, Visualizer, 
+                 TFOpLambda, Lambda, SlicingOpLambda, Repeat, Variable, Gather]
