@@ -1,7 +1,9 @@
 import json
 from collections import defaultdict
 from pathlib import Path
-from typing import Set, Dict, Any, List
+from typing import Set, Dict, Any, List, Type, Optional
+import tensorflow as tf  # type: ignore
+from tensorflow.python.keras.utils.generic_utils import func_load
 
 from leap_model_parser.contract.importmodelresponse import NodeResponse, WrapperData
 
@@ -9,13 +11,17 @@ layer_attributes_adaptation = {'bias_initializer', 'kernel_initializer'}
 
 
 class KerasJsonModelImport:
-    def __init__(self) -> None:
+    def __init__(self, custom_layers: Optional[Dict[str, Type[tf.keras.layers.Layer]]] = None) -> None:
         self.id = 1
         self.nodes_cache: Dict[str, NodeResponse] = defaultdict()
         self.layer_name_to_layer: Dict[str, Dict] = defaultdict()
         self.visited_connections: Set[str] = set()
         self.ui_components: Dict[str, Dict] = defaultdict()
 
+        if custom_layers is None:
+            custom_layers = {}
+        self.custom_layers = custom_layers
+
     def prepare_model_to_import(self, model_schema) -> None:
         ui_components_file_path = Path(__file__).parent / 'contract' / 'ui_components.json'
         with open(str(ui_components_file_path), 'r') as f:
@@ -37,12 +43,12 @@ class KerasJsonModelImport:
             current_layer = self.layer_name_to_layer[graph_output_layer[0]]
             self.generate_graph_recursive(current_layer, graph_output_layer[1])
 
-        nodes = {node.id: node for node in self.nodes_cache.values() if not node.name == 'Lambda'}
+        nodes = {node.id: node for node in self.nodes_cache.values()}
         _add_origin_name_to_nodes(nodes)
         return nodes
 
     '''
-    Convert tf ops inbound nodes to the same format as the regular layers.
+    Convert tf ops inbound nodes to the same format as the regular layers, while removing _CONSTANT_VALUE.
     examples:
     1.
         ['conv2d_2', 0, 0, {'slice_spec': [{'start': None, 'stop': None, 'step': None}]}
@@ -52,22 +58,28 @@ class KerasJsonModelImport:
         ['tf.__operators__.getitem_1', 0, 0, {'y': ['tf.__operators__.getitem_1', 0, 0]}]
         will be converted to:
         [['tf.__operators__.getitem_1', 0, 0], ['tf.__operators__.getitem_1', 0, 0]]
+    3.
+        ['_CONSTANT_VALUE', -1, 1.0, {'y': ['tf.__operators__.add_2', 0, 0], 'name': None}]
+        will be converted to:
+        [['tf.__operators__.add_2', 0, 0]]
     '''
+
     @staticmethod
-    def prepare_inbound_nodes_of_tf_ops(inbound_nodes_to_enumerate: List):
+    def prepare_inbound_nodes(inbound_nodes_to_enumerate: List):
         inbound_nodes_to_enumerate_prepared = []
         single_inbound_node_to_enumerate = []
         for element in inbound_nodes_to_enumerate:
             if type(element) in (int, str):
                 single_inbound_node_to_enumerate.append(element)
             elif isinstance(element, dict):
-                if single_inbound_node_to_enumerate and type(single_inbound_node_to_enumerate[0]) == str:
+                if single_inbound_node_to_enumerate and type(single_inbound_node_to_enumerate[0]) == str and \
+                        single_inbound_node_to_enumerate[0] != '_CONSTANT_VALUE':
                     inbound_nodes_to_enumerate_prepared.append(single_inbound_node_to_enumerate)
                 single_inbound_node_to_enumerate = []
                 for inbound_nodes in element.values():
                     if isinstance(inbound_nodes, list):
                         inbound_nodes_to_enumerate_prepared.extend(
-                            KerasJsonModelImport.prepare_inbound_nodes_of_tf_ops(inbound_nodes))
+                            KerasJsonModelImport.prepare_inbound_nodes(inbound_nodes))
         if single_inbound_node_to_enumerate and type(single_inbound_node_to_enumerate[0]) == str:
             inbound_nodes_to_enumerate_prepared.append(single_inbound_node_to_enumerate)
 
@@ -80,10 +92,14 @@ class KerasJsonModelImport:
         if len(inbound_nodes) == 0:
             return
         inbound_nodes_to_enumerate = inbound_nodes[instance_index]
-        if not isinstance(inbound_nodes_to_enumerate[0], list):
-            inbound_nodes_to_enumerate = self.prepare_inbound_nodes_of_tf_ops(inbound_nodes_to_enumerate)
+        if isinstance(inbound_nodes_to_enumerate[0], list):
+            inbound_nodes_to_enumerate_prepared = []
+            for inbound_node in inbound_nodes_to_enumerate:
+                inbound_nodes_to_enumerate_prepared.extend(self.prepare_inbound_nodes(inbound_node))
+        else:
+            inbound_nodes_to_enumerate_prepared = self.prepare_inbound_nodes(inbound_nodes_to_enumerate)
 
-        for input_index, input_layer_item in enumerate(inbound_nodes_to_enumerate):
+        for input_index, input_layer_item in enumerate(inbound_nodes_to_enumerate_prepared):
             input_layer = self.layer_name_to_layer[input_layer_item[0]]
             self.generate_node(input_layer, input_layer_item[1])
 
@@ -122,6 +138,16 @@ class KerasJsonModelImport:
             "output": node_output_key
         })
 
+    def _get_layer_metadata(self, layer_json: Dict) -> Optional[Dict]:
+        layer_metadata = self.ui_components.get(layer_json['class_name'])
+        if layer_metadata is None:
+            if layer_json['class_name'] in self.custom_layers:
+                layer_metadata = {
+                    'type': 'UserCustomLayer',
+                    'name': layer_json['class_name']
+                }
+        return layer_metadata
+
     def generate_node(self, layer: Dict[str, Any], instance_index: int):
         if layer['class_name'] == 'InputLayer':
             self.handle_input_layer(layer)
@@ -132,7 +158,7 @@ class KerasJsonModelImport:
             return
 
         layer = self.handle_cudnn(layer)
-        layer_metadata = self.ui_components.get(layer['class_name'])
+        layer_metadata = self._get_layer_metadata(layer)
         if layer_metadata is None:
             raise Exception(f"{layer['class_name']} layer is not supported, please try to contact Tensorleap support")
 
@@ -151,6 +177,9 @@ class KerasJsonModelImport:
         data = layer['config']
         if layer['class_name'] in ('TFOpLambda', 'SlicingOpLambda'):
             data['inbound_nodes'] = layer['inbound_nodes'][0]
+        elif layer['class_name'] == "Lambda":
+            lambda_func = func_load(layer['config']['function'])
+            data['arg_names'] = list(lambda_func.__code__.co_varnames)
 
         self.layer_data_adjustments(data, layer_metadata)
         node = NodeResponse(id=str(self.id), name=layer["class_name"], data=data)
@@ -175,9 +204,6 @@ class KerasJsonModelImport:
 
     @classmethod
     def layer_data_adjustments(cls, layer_config: Dict[str, Any], layer_metadata: Dict[str, Any]) -> None:
-        if layer_metadata['name'] == 'Lambda':
-            layer_config.pop('function')
-
         for layer_attribute in layer_attributes_adaptation:
             if layer_attribute in layer_config:
                 layer_config[layer_attribute] = layer_config[layer_attribute]['class_name']
@@ -220,7 +246,10 @@ class KerasJsonModelImport:
         elif node.name == 'Dataset':
             connection_format = f"{node.id}-{node_input_layer['name']}"
         else:
-            connection_format = layer_metadata[f"{direction}_data"][direction][0]['name']
+            try:
+                connection_format = layer_metadata[f"{direction}_data"][direction][index]['name']
+            except IndexError:
+                connection_format = layer_metadata[f"{direction}_data"][direction][0]['name']
             if connection_format.count('${id}') > 0:
                 connection_format = f"{node.id}-{index}"
                 if 'custom_input_keys' not in node.data:
@@ -269,6 +298,7 @@ def _add_origin_name_to_nodes(nodes: Dict[str, NodeResponse]):
 
 
 DATASET_DATA = {'datasetVersion': {'metadata': {'setup': {'preprocess': {'training_length': 0, 'validation_length': 0},
-                'inputs': [], 'metadata': [], 'outputs': [], 'visualizers': [], 'prediction_types': [],
-                'custom_loss_names': []}}},
+                                                          'inputs': [], 'metadata': [], 'outputs': [],
+                                                          'visualizers': [], 'prediction_types': [],
+                                                          'custom_loss_names': []}}},
                 'type': 'dataset'}
