@@ -3,7 +3,8 @@ import glob
 import json
 import tempfile
 from pathlib import Path
-from typing import Callable, Optional, Dict, Tuple
+from typing import Callable, Optional, Dict, Tuple, Type
+import tensorflow as tf  # type: ignore
 
 from keras import Model  # type: ignore
 from onnx2kerastl.exceptions import OnnxUnsupported  # type: ignore
@@ -11,7 +12,7 @@ from tensorflow.keras.models import load_model  # type: ignore
 import tarfile
 import ntpath
 from onnx2kerastl import onnx_to_keras  # type: ignore
-from leap_model_rebuilder import rebuild_model # type: ignore
+from leap_model_rebuilder import rebuild_model  # type: ignore
 from keras_data_format_converter import convert_channels_first_to_last  # type: ignore
 
 from leap_model_parser.contract.importmodelresponse import NodeResponse, ImportModelTypeEnum
@@ -26,23 +27,15 @@ if spec is not None:
     onnx_imported = True
 
 
-def get_k_model_from_pb_path(file_path):
-    tf = tarfile.open(file_path)
-    with tempfile.TemporaryDirectory() as temp_dir:
-        tf.extractall(temp_dir)
-        pb_files = glob.glob(temp_dir + "/**/*.pb", recursive=True)
-        if len(pb_files) == 0:
-            raise Exception('no pb files were found')
-
-        pb_file_path = next(iter(pb_files))
-        pb_folder_path = next(iter(ntpath.split(pb_file_path)))
-        k_model = load_model(pb_folder_path)
-    return k_model
-
 
 class ModelParser:
-    def __init__(self):
-        self._should_transform_inputs = False
+    def __init__(self, should_transform_inputs=False,
+                 custom_layers: Optional[Dict[str, Type[tf.keras.layers.Layer]]] = None):
+        self._should_transform_inputs = should_transform_inputs
+
+        self.custom_layers = custom_layers
+        if custom_layers is None:
+            self.custom_layers = {}
 
         self._model_types_converter = {
             ImportModelTypeEnum.JSON_TF2.value: self.convert_json_model,
@@ -51,38 +44,50 @@ class ModelParser:
             ImportModelTypeEnum.PB_TF2.value: self.convert_pb_model,
         }
 
-    def get_keras_model_and_model_graph(self, model_path: Path, model_type: ImportModelTypeEnum,
-                                        should_transform_inputs=False) -> Tuple[Dict[str, NodeResponse], Optional[Model]]:
-
-        self._should_transform_inputs = should_transform_inputs
-        model_to_keras_converter: Optional[Callable[[str], Dict]] = self._model_types_converter.get(model_type.value)
+    def get_keras_model_and_model_graph(
+            self, model_path: Path, model_type: ImportModelTypeEnum) -> Tuple[Dict[str, NodeResponse], Optional[Model]]:
+        model_to_keras_converter: Optional[Callable[[str], Tuple[Dict[str, NodeResponse], Optional[Model]]]] = \
+            self._model_types_converter.get(model_type.value)
         if model_to_keras_converter is None:
             raise Exception(f"Unable to import external version, {str(model_path)} file format isn't supported")
 
         file_path = str(model_path)
         try:
             model_schema, keras_model = model_to_keras_converter(file_path)
-            model_generator = KerasJsonModelImport()
+            model_generator = KerasJsonModelImport(self.custom_layers)
             graph = model_generator.generate_graph(model_schema)
             return graph, keras_model
         except Exception as e:
             if model_type.value in [ImportModelTypeEnum.H5_TF2.value, ImportModelTypeEnum.PB_TF2.value]:
                 if model_type.value == ImportModelTypeEnum.H5_TF2.value:
-                    keras_model = load_model(file_path)
+                    keras_model = self._load_keras_model_with_custom_layers(file_path)
                 else:
-                    keras_model = get_k_model_from_pb_path(file_path)
+                    keras_model = self._get_k_model_from_pb_path(file_path)
 
                 rebuilt_model = rebuild_model(keras_model)
                 model_schema, keras_model = self.convert_to_keras_model(rebuilt_model)
-                model_generator = KerasJsonModelImport()
+                model_generator = KerasJsonModelImport(self.custom_layers)
                 graph = model_generator.generate_graph(model_schema)
                 return graph, keras_model
             else:
                 raise e
 
-    def generate_model_graph(self, model_path: Path, model_type: ImportModelTypeEnum,
-                             should_transform_inputs=False) -> Dict[str, NodeResponse]:
-        model_graph, _ = self.get_keras_model_and_model_graph(model_path, model_type, should_transform_inputs)
+    def _get_k_model_from_pb_path(self, file_path: str):
+        tar_file = tarfile.open(file_path)
+        with tempfile.TemporaryDirectory() as temp_dir:
+            tar_file.extractall(temp_dir)
+            pb_files = glob.glob(temp_dir + "/**/*.pb", recursive=True)
+            if len(pb_files) == 0:
+                raise Exception('no pb files were found')
+
+            pb_file_path = next(iter(pb_files))
+            pb_folder_path = next(iter(ntpath.split(pb_file_path)))
+
+            k_model = self._load_keras_model_with_custom_layers(pb_folder_path)
+        return k_model
+
+    def generate_model_graph(self, model_path: Path, model_type: ImportModelTypeEnum) -> Dict[str, NodeResponse]:
+        model_graph, _ = self.get_keras_model_and_model_graph(model_path, model_type)
         return model_graph
 
     @classmethod
@@ -92,7 +97,7 @@ class ModelParser:
         return model_schema, None
 
     def convert_pb_model(self, file_path: str) -> Tuple[Dict[str, NodeResponse], Model]:
-        k_model = get_k_model_from_pb_path(file_path)
+        k_model = self._get_k_model_from_pb_path(file_path)
         return self.convert_to_keras_model(k_model)
 
     def convert_onnx_model(self, file_path: str) -> Tuple[Dict[str, NodeResponse], Model]:
@@ -106,9 +111,14 @@ class ModelParser:
 
         return self.convert_to_keras_model(k_model)
 
-    def convert_h5_model(self, file_path: str) -> Tuple[Dict[str, NodeResponse], Model]:
-        imported_model = load_model(file_path)
+    def _load_keras_model_with_custom_layers(self, file_path: str):
+        custom_objects = {}
+        if self.custom_layers is not None:
+            custom_objects = self.custom_layers
+        return load_model(file_path, custom_objects=custom_objects)
 
+    def convert_h5_model(self, file_path: str) -> Tuple[Dict[str, NodeResponse], Model]:
+        imported_model = self._load_keras_model_with_custom_layers(file_path)
         return self.convert_to_keras_model(imported_model)
 
     def convert_to_keras_model(self, k_model) -> Tuple[Dict[str, NodeResponse], Model]:
